// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "PlayerController.hpp"
#include "game/DawnGame.hpp"

using namespace Dawn;

PlayerController::PlayerController(SceneItem *item) : SceneItemComponent(item) {

}

std::vector<SceneItemComponent*> PlayerController::getDependencies() {
  return {
    (this->characterController = item->getComponent<CharacterController2D>()),
    (this->entityHealth = item->getComponent<EntityHealth>()),
    (this->entityMove = item->getComponent<EntityMove>())
  };
}

void PlayerController::onStart() {
  assertNotNull(this->characterController);
  assertNotNull(this->entityMove);
  assertNotNull(this->entityHealth);

  useEvent([&](float_t delta){
    // Movement
    this->entityMove->direction = getGame()->inputManager.getAxis2D(
      INPUT_BIND_NEGATIVE_X, INPUT_BIND_POSITIVE_X,
      INPUT_BIND_NEGATIVE_Y, INPUT_BIND_POSITIVE_Y
    );
  }, getScene()->eventSceneUpdate);

  // Hurt Hazard Processing
  useEvent([&](struct CharacterController2DCollisionEventInfo info) {
    if(entityHealth->isInvincible()) return;

    auto hurtHazard = info.collider->item->getComponent<HurtHazard>();
    if(hurtHazard == nullptr) return;

    glm::vec2 back = glm::vec2(
      info.collider->transform->getLocalPosition().x,
      info.collider->transform->getLocalPosition().z
    ) - glm::vec2(
      this->transform->getLocalPosition().x,
      this->transform->getLocalPosition().z
    );

    this->characterController->velocity = back * -hitKnockback;
    this->entityHealth->damage({
      .amount = hurtHazard->damage
    });
  }, this->characterController->eventCollision);

}