// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "EntityChargePlayer.hpp"

using namespace Dawn;

EntityChargePlayer::EntityChargePlayer(SceneItem* item) : SceneItemComponent(item) {
  
}

std::vector<SceneItemComponent*> EntityChargePlayer::getDependencies() {
  return {
    (characterController = item->getComponent<CharacterController2D>())
  };
}

void EntityChargePlayer::onStart() {
  assertNotNull(characterController);

  useEvent([&](float_t delta) {
    // Get the direction to move in
    PlayerController * player = getScene()->findComponent<PlayerController>();
    if(player == nullptr) return;
    
    // Get the direction to move in
    glm::vec2 pos = glm::vec2(
      player->transform->getLocalPosition().x,
      player->transform->getLocalPosition().z
    ) - glm::vec2(
      transform->getLocalPosition().x,
      transform->getLocalPosition().z
    );

    // Conver to an angle and then convert back into a 2D vector. This mimics a
    // user essentially "moving the analogue stick in this direction"
    
    if(mathAbs(pos.x) > 1.0f || mathAbs(pos.y) > 1.0f) {
      float_t angle = atan2(pos.y, pos.x);
      glm::vec2 movement(cos(angle), sin(angle));
      transform->setLocalRotation(glm::quat(glm::vec3(0, -angle + 1.5708f, 0)));
      characterController->velocity += movement * delta * moveSpeed;
    }
  }, getScene()->eventSceneUpdate);
}